@@ -24,9 +24,7 @@
 #include <glib-object.h>
 #include <glib/gi18n.h>
 
-#include <nm-utils.h>
-#include <nm-device-wifi.h>
-#include <nm-device-ethernet.h>
+#include <NetworkManager.h>
 
 #include "../panel-common.h"
 #include "ce-page-details.h"
@@ -141,8 +139,8 @@ connect_details_page (CEPageDetails *page)
                 const gchar *p1, *p2;
 
                 ac = nm_device_get_active_connection (page->device);
-                p1 = ac ? nm_active_connection_get_connection (ac) : NULL;
-                p2 = nm_connection_get_path (CE_PAGE (page)->connection);
+                p1 = ac ? nm_active_connection_get_uuid (ac) : NULL;
+                p2 = nm_connection_get_uuid (CE_PAGE (page)->connection);
                 if (g_strcmp0 (p1, p2) == 0) {
                         device_is_active = TRUE;
                         if (NM_IS_DEVICE_WIFI (page->device))
@@ -226,7 +224,6 @@ ce_page_details_class_init (CEPageDetailsClass *class)
 CEPage *
 ce_page_details_new (NMConnection     *connection,
                      NMClient         *client,
-                     NMRemoteSettings *settings,
                      NMDevice         *device,
                      NMAccessPoint    *ap)
 {
@@ -235,7 +232,6 @@ ce_page_details_new (NMConnection     *connection,
         page = CE_PAGE_DETAILS (ce_page_new (CE_TYPE_PAGE_DETAILS,
                                              connection,
                                              client,
-                                             settings,
                                              "/org/gnome/control-center/network/details-page.ui",
                                              _("Details")));
 
