@@ -104,14 +104,6 @@ get_os_name (void)
        g_free (buffer);
     }
 
-  if (name && *name != '\0')
-    {
-      char *tmp;
-      tmp = g_shell_unquote (name, NULL);
-      g_free (name);
-      name = tmp;
-    }
-
   if (name == NULL)
     name = g_strdup ("GNOME");
 
@@ -145,14 +137,6 @@ get_privacy_policy_url (void)
        g_free (buffer);
     }
 
-  if (name && *name != '\0')
-    {
-      char *tmp;
-      tmp = g_shell_unquote (name, NULL);
-      g_free (name);
-      name = tmp;
-    }
-
   if (name == NULL)
     name = g_strdup ("http://www.gnome.org/privacy-policy");
 
