@@ -679,9 +679,9 @@ cc_sharing_panel_setup_label (CcSharingPanel *self,
   if (label == WID ("personal-file-sharing-label"))
     text = g_strdup_printf (_("Personal File Sharing allows you to share your Public folder with others on your current network using: <a href=\"dav://%s\">dav://%s</a>"), hostname, hostname);
   else if (label == WID ("remote-login-label"))
-    text = g_strdup_printf (_("Allow remote users to connect using the Secure Shell command:\n<a href=\"ssh %s\">ssh %s</a>"), hostname, hostname);
+    text = g_strdup_printf (_("When remote login is enabled, remote users can connect using the Secure Shell command:\n<a href=\"ssh %s\">ssh %s</a>"), hostname, hostname);
   else if (label == WID ("screen-sharing-label"))
-    text = g_strdup_printf (_("Allow remote users to view or control your screen by connecting to: <a href=\"vnc://%s\">vnc://%s</a>"), hostname, hostname);
+    text = g_strdup_printf (_("Screen sharing allows remote users to view or control your screen by connecting to <a href=\"vnc://%s\">vnc://%s</a>"), hostname, hostname);
   else
     g_assert_not_reached ();
 
@@ -1013,7 +1013,7 @@ cc_sharing_panel_setup_screen_sharing_dialog (CcSharingPanel *self)
 
   /* settings bindings */
   settings = g_settings_new (VINO_SCHEMA_ID);
-  g_settings_bind (settings, "view-only", WID ("remote-control-switch"),
+  g_settings_bind (settings, "view-only", WID ("remote-control-checkbutton"),
                    "active",
                    G_SETTINGS_BIND_DEFAULT | G_SETTINGS_BIND_INVERT_BOOLEAN);
   g_settings_bind (settings, "prompt-enabled",
